from kfpubsub import PubSub
from django.conf import settings
from django.db import close_old_connections


class DjangoPubSub(PubSub):

    def __init__(self):
        host = getattr(settings, 'REDIS_HOST', 'localhost')
        port = getattr(settings, 'REDIS_PORT', 6379)
        database = getattr(settings, 'REDIS_DB', 0)

        enable_kafka = getattr(settings, 'KAFKA_ENABLE', False)
        if enable_kafka:
            kafka_brokers = getattr(settings, 'KAFKA_BROKERS', ("localhost:9092", ))
            kafka_group_id = getattr(settings, 'KAFKA_GROUP_ID', None)
            kafka_enable_auto_commit = getattr(settings, 'KAFKA_ENABLE_AUTO_COMMIT', False)
            kafka_topics = getattr(settings, 'KAFKA_TOPICS', None)
            kafka_replication_factor = getattr(settings, 'KAFKA_REPLICATION_FACTOR', 1)
            kafka_partitions = getattr(settings, 'KAFKA_PARTITIONS', 1)
            kafka_poll_timeout = getattr(settings, 'KAFKA_POLL_TIMEOUT', 500)
            kafka_emit_prefix = getattr(settings, 'KAFKA_EMIT_PREFIX', '')
            kafka_receive_prefix = getattr(settings, 'KAFKA_RECEIVE_PREFIX', '')

            super(DjangoPubSub, self).__init__(
                host=host,
                port=port,
                database=database,
                enable_kafka=enable_kafka,
                kafka_brokers=kafka_brokers,
                kafka_group_id=kafka_group_id,
                kafka_enable_auto_commit=kafka_enable_auto_commit,
                kafka_partitions=kafka_partitions,
                kafka_replication_factor=kafka_replication_factor,
                kafka_topics=kafka_topics,
                kafka_poll_timeout=kafka_poll_timeout,
                kafka_emit_prefix=kafka_emit_prefix,
                kafka_receive_prefix=kafka_receive_prefix,
            )
        else:
            super(DjangoPubSub, self).__init__(host=host, port=port, database=database)

    def emit(self, event, message):
        super(DjangoPubSub, self).emit(event, message, emit=getattr(settings, 'PUB_SUB_EMIT', True))

    def receive(self, listeners):
        super(DjangoPubSub, self).receive(listeners, close_old_connections)



from kfpubsub.utils import HandlerManager
from django.conf import settings


class Handler(HandlerManager):

    def __init__(self):
        super(Handler, self).__init__(
            getattr(settings, 'BASE_DIR', ''),
            getattr(settings, 'EVENT_HANDLERS_DIR_NAME', 'event_handlers') ,
            getattr(settings, 'HANDLER_FILES_PREFIX', 'handlers_')
        )



# -*- coding: utf-8 -*-
import logging

from django.core.management.base import BaseCommand
from django.utils.translation import ugettext as _
from redis.exceptions import ConnectionError

from djangopubsub.djangoPubSub import DjangoPubSub as PubSub
from djangopubsub.handler import Handler as HandlerManager

logger = logging.getLogger(__name__)
logging.getLogger().setLevel(logging.INFO)


class Command(BaseCommand):
    help = _(u'Running Receiver PubSub!')
    error = False

    def runReceiverPubSub(self):
        try:
            events_map = HandlerManager().get_events_mapping()
            pubsub = PubSub()
            pubsub.receive(events_map)

        except ConnectionError as e:
            logger.error(str(e))

        except Exception as e:
            self.error = True
            logger.error(str(e), exc_info=True)

    def handle(self, *args, **options):
        logging.info('''------------------------------ Running PubSub --------------------------------
                        This is gonna take a few minutes...''')
        while not self.error:
            self.runReceiverPubSub()

        logging.info('''------------------------------ Finnished --------------------------------''')



from setuptools import setup, find_packages
from os import path

current_dir = path.abspath(path.dirname(__file__))

with open(path.join(current_dir, 'README.md')) as f:
    long_description = f.read()

setup(name='djangopubsub',
      version='0.3.4',
      description='Base redis pubsub django wrapper',
      long_description=long_description,
      long_description_content_type='text/markdown',
      url='https://gitlab.com/kas-factory/packages/django-pubsub',
      author='Antonio @ KF',
      author_email='antonio@kasfactory.net',
      license='COPYRIGHT',
      packages=find_packages(),
      package_data={'djangopubsub': ['djangopubsub/*',
                                     'djangopubsub/management/*',
                                     'djangopubsub/management/commands/*']},
      install_requires=[
            'Django>=1.8.18',
            'kfpubsub>=0.3.4'
      ],
      zip_safe=False
)
